ProofHash

Type Alias ProofHash 

pub type ProofHash = [u8; 32];
Hash used in a merkle proof. If it’s a leaf node, its a vote reference, otherwise a node hash.

The node hash of a leaf node can be derived from its vote reference.